"""Module for output to console"""
from rich.console import Console
from rich.table import Table
import typer

console = Console()

ERROR_STYLE = "bold red"
INFO_STYLE = "bold blue"
SUCCESS_STYLE = "bold green"


def success_message(message: str) -> None:
    """Prints a success message"""
    show_message(message=message, style=SUCCESS_STYLE)


def error_message(message: str) -> None:
    """Prints an error message"""
    show_message(message=message, style=ERROR_STYLE)


def show_table(table: Table) -> None:
    """Prints a table"""
    console.print(table)


def show_message(message: str, style: str = None) -> None:
    """Prints a message"""
    if style is None:
        style = INFO_STYLE
    console.print(f"{message}", style=style)


def abort(message: str):
    """Prints an error message"""
    error_message(message=message)
    raise typer.Abort()



"""PDF set commads."""
from typing import Optional
import typer
from toolbox.sets.pdf.converter import convert_pdf_to_docx, convert_pdf_to_selectable
from toolbox.common.output import show_message, abort
from toolbox.common.exceptions import ConvertPDFException
from toolbox.sets.pdf import __version__ as pdf_package_version

app = typer.Typer(short_help="PDF CLI Tool")


def version_callback(value: bool):
    """Version callback."""
    if value:
        show_message(f"PDF CLI Version: {pdf_package_version.__version__}")
        raise typer.Exit()


@app.command()
def convert_to_selectable_pdf(input_file: str, output_file: Optional[str] = typer.Argument(None)):
    """Converts a scan PDF file to selectable PDF."""
    try:
        convert_pdf_to_selectable(input_file, output_file)
    except ConvertPDFException as err:
        abort(f'Error converting to selectable pdf: {err}')
    show_message("File converted successfully[green].")


@app.command()
def convert_to_docx(input_file: str, output_file: Optional[str] = typer.Argument(None)):
    """Converts a PDF file to a DOCX file."""
    try:
        convert_pdf_to_docx(input_file, output_file)
    except ConvertPDFException as err:
        abort(f'Error converting to docx: {err}')
    show_message("File converted successfully[green].")


@app.callback()
def version(ver: bool = typer.Option(None, "--version",
                                     callback=version_callback,
                                     help="Get command version.",
                                     is_eager=True)):
    """Version Output."""
    return ver



"""Download class"""
import urllib.request
import sys
import time
from os import path, get_terminal_size, name
import itertools
from re import match
from toolbox.common.exceptions import DownloadException


class Download:

    def __init__(
        self,
        url,
        des=None,
        overwrite=False,
        continue_download=False,
        echo=False,
        quiet=False,
        batch=False,
        icon_done="▓",
        icon_left="░",
        icon_border="|"
    ):
        self.url = url
        self.des = des
        self.passed_dir = None
        self.headers = {}
        self.f_size = 0
        self.done_icon = icon_done if len(icon_done) < 2 else "▓"
        self.left_icon = icon_left if len(icon_left) < 2 else "░"
        self.border_left, self.border_right = self._extract_border_icon(
            icon_border)
        self._cycle_bar = None
        self.echo = echo
        self.quiet = quiet
        self.batch = batch
        self.overwrite = overwrite
        self.continue_download = continue_download
        self.file_exists = False
        self.ostream = sys.stderr if self.echo else sys.stdout
        self.basename = None
        self.conn = None
        self.req = None

    def _extract_border_icon(self, passed_icon):
        """"
        Extract the passed border icon according to
        what is passed.
        If the string has length equal to 2, then use the
        first char as left border icon and the second as
        right.
        If the string has length equal to 1, use the same icon for both.
        """
        if len(passed_icon) == 1:
            return passed_icon, passed_icon

        if len(passed_icon) == 2:
            return passed_icon[0], passed_icon[1]

        return "|", "|"

    def _build_headers(self, rem):
        """Build headers according to requirement."""
        self.headers = {"Range": f"bytes={rem}-"}
        print(f"Trying to resume download at: {rem} bytes", file=self.ostream)

    def _parse_exists(self):
        """This function should be called if the file already exists.
        In that case there are two possibilities, it's partially downloaded
        or it's a proper file.
        """
        if self.overwrite:
            return
        if self.continue_download:
            cur_size = path.getsize(self.des)
            original_size = urllib.request.urlopen(self.url).info()[
                'Content-Length']

            if original_size is None:
                print("WARNING: Could not perform sanity check on partial download.",
                      file=self.ostream)
                self._build_headers(cur_size)
            elif cur_size < int(original_size):
                self._build_headers(cur_size)
        else:
            print("ERROR: File exists. See 'dw --help' for solutions.",
                  file=self.ostream)
            sys.exit(-1)

    def _preprocess_conn(self):
        """Make necessary things for the connection."""
        self.req = urllib.request.Request(url=self.url, headers=self.headers)
        self.conn = urllib.request.urlopen(self.req)
        self.f_size = self.conn.info()['Content-Length']
        if self.f_size is not None:
            self.f_size = int(self.f_size)

    def _get_terminal_length(self):
        """Return the length of the terminal."""
        # If quiet is passed, skip this calculation and return a default length
        if self.quiet:
            return 50

        cols = get_terminal_size().columns
        return cols if name != "nt" else cols - 1

    def _parse_destination(self):
        # Check if the des is passed
        if self.des is not None:
            if path.isdir(self.des):
                self.passed_dir = self.des
                self.des = path.join(self.des, self._get_name())
        else:
            self.des = self._get_name()

        # Put a check to see if file already exists.
        # Try to resume it if that's true
        if path.exists(self.des):
            self._parse_exists()
            self.file_exists = True

    def _is_valid_src_path(self, file_path):
        """Check to see if the path passed is
        a valid source path.
        A valid source path would be a file that
        is not a directory and actually a file
        present in the disk.
        """
        return not path.exists(file_path) or not path.isfile(file_path)

    def _parse_url(self):
        """
        The URL can be a file as well so in that case we
        will download each URL from that file.
        In case the URL is not a file and just a simple URL,
        download just that one.
        returns: A list of urls
        """
        if match(r"^https?://*|^file://*", self.url):
            return [self.url]

        # Below code will only be executed if the -b
        # flag is passed
        if not self.batch:
            print(f"{self.url}: not a valid URL. Pass -b if it is a file "
                  "containing various URL's and you want bulk download.")
            sys.exit(0)

        rel_path = path.expanduser(self.url)

        # Put a check to see if the file is present
        if self._is_valid_src_path(rel_path):
            print(f"{rel_path}: not a valid name or is a directory")
            sys.exit(-1)

        # If it's not an URL, read the contents.
        # Since the URL is not an actual URL, we're assuming
        # it is a file that contains URL's seperated by new
        # lines.
        with open(rel_path, "r", encoding="utf8") as rstream:
            return rstream.read().split("\n")

    def _get_name(self):
        """Try to get the name of the file from the URL."""

        file_name = 'temp'
        temp_url = self.url

        split_url = temp_url.split('/')[-1]

        if split_url:
            # Remove query params if any
            file_name = split_url.split("?")[0]

        return file_name

    def _format_size(self, size):
        """Format the passed size.
        If its more than an 1 Mb then return the size in Mb's
        else return it in Kb's along with the unit.
        """
        map_unit = {0: 'bytes', 1: "KB", 2: "MB", 3: "GB"}
        formatted_size = size

        no_iters = 0
        while formatted_size > 1024:
            no_iters += 1
            formatted_size /= 1024

        return (formatted_size, map_unit[no_iters])

    def _format_time(self, time_left):
        """Format the passed time depending."""
        unit_map = {0: 's', 1: 'm', 2: 'h', 3: 'd'}

        no_iter = 0
        while time_left > 60:
            no_iter += 1
            time_left /= 60

        return time_left, unit_map[no_iter]

    def _format_speed(self, speed):
        """Format the speed."""
        unit = {0: 'Kb/s', 1: 'Mb/s', 2: 'Gb/s'}

        inc_with_iter = 0
        while speed > 1000:
            speed = speed / 1000
            inc_with_iter += 1

        return speed, unit[inc_with_iter]

    def _get_speed_n_time(self, file_size_dl, beg_time, cur_time):
        """Return the speed and time depending on the passed arguments."""

        # Sometimes the beg_time and the cur_time are same, so we need
        # to make sure that doesn't raise a ZeroDivisionError in the
        # following line.
        if cur_time == beg_time:
            return "Inf", "", 0, ""

        # Calculate speed
        speed = (file_size_dl / 1024) / (cur_time - beg_time)

        # Calculate time left
        if self.f_size is not None:
            time_left = ((self.f_size - file_size_dl) / 1024) / speed
            time_left, time_unit = self._format_time(time_left)
        else:
            time_left, time_unit = 0, ""

        # Format the speed
        speed, s_unit = self._format_speed(speed)

        return round(speed), s_unit, round(time_left), time_unit

    def _get_pos(self, reduce_with_each_iter):
        if self._cycle_bar is None:
            self._cycle_bar = itertools.cycle(
                range(0, int(reduce_with_each_iter)))

        return next(self._cycle_bar) + 1

    def _get_bar(self, status, length, percent=None):
        """Calculate the progressbar depending on the length of terminal."""
        # Till now characters present is the length of status.
        # length is the length of terminal.
        # We need to decide how long our bar will be.
        cur_len = len(status) + 2  # 2 for bar

        if percent is not None:
            cur_len += 5  # 5 for percent

        reduce_with_each_iter = 40
        while reduce_with_each_iter > 0:
            if cur_len + reduce_with_each_iter > length:
                reduce_with_each_iter = int(reduce_with_each_iter / 2)
            else:
                break

        # Add space.
        space = length - (len(status) + 2 + reduce_with_each_iter + 5)
        status += fr'{" " * space}'

        if reduce_with_each_iter > 0:
            # Make BOLD
            status += "\033[1m"
            # Add color.
            status += "\033[1;34m"
            if percent is not None:
                done = int(percent / (100 / reduce_with_each_iter))
                status += fr"{self.border_left}{self.done_icon * done}\
                {self.left_icon * (reduce_with_each_iter - done)}{self.border_right}"
            else:
                current_pos = self._get_pos(reduce_with_each_iter)
                status_bar = " " * (current_pos - 1) if current_pos > 1 else ""
                status_bar += self.done_icon * 1
                status_bar += " " * int(reduce_with_each_iter - current_pos)
                status += fr"{self.border_left}{status_bar}{self.border_right}"

        status += "\033[0m"
        return status

    def _download(self):
        try:
            self._parse_destination()
            self._preprocess_conn()
            with open(self.des, 'ab') as wstream:
                if self.f_size is not None and self.quiet is False:
                    formatted_file_size, dw_unit = self._format_size(self.f_size)
                    print(f"Size: {round(formatted_file_size)} {dw_unit}", file=self.ostream)
                _owrite = (f"Overwriting: {self.des}" if (self.file_exists and self.overwrite)
                           else f"Saving as: {self.des}")
                if self.quiet:
                    self.ostream.write(_owrite)
                    self.ostream.write("...")
                else:
                    print(_owrite, file=self.ostream)
                    self.ostream.flush()
                file_size_dl = 0
                block_sz = 8192
                beg_time = time.time()
                while True:
                    buffer = self.conn.read(block_sz)
                    if not buffer:
                        break

                    file_size_dl += len(buffer)
                    wstream.write(buffer)
                    percent = ''

                    speed, s_unit, time_left, time_unit = self._get_speed_n_time(
                        file_size_dl,
                        beg_time,
                        cur_time=time.time()
                    )

                    if self.f_size is not None:
                        percent = file_size_dl * 100 / self.f_size

                    # Get basename
                    self.basename = path.basename(self.des)

                    # Calculate amount of space req in between
                    length = self._get_terminal_length()

                    f_size_disp, dw_unit = self._format_size(file_size_dl)

                    status = fr"{round(f_size_disp)} {dw_unit}-7s"
                    status += fr'| {speed}-3s {s_unit}'

                    if self.f_size is not None:
                        status += fr"|| ETA: {time_left} {time_unit}-4s "
                        status = self._get_bar(status, length, percent)
                        status += fr" {round(percent)}-4s"
                    else:
                        status = self._get_bar(status, length)

                    if not self.quiet:
                        self.ostream.write('\r')
                        self.ostream.write(status)
                        self.ostream.flush()

            wstream.close()
            if self.quiet:
                self.ostream.write("...success\n")
                self.ostream.flush()

        except KeyboardInterrupt:
            self.ostream.flush()
            print("Keyboard Interrupt passed. Exiting peacefully.")
            sys.exit(0)
        except DownloadException as exception:
            print(f'ERROR: {exception}')
            return False
        return True

    def is_quiet(self):
        """Return if download is quiet."""
        return self.quiet

    def download(self):
        """
        download will iterate through a list of possible url's
        and destinations and keep passing to the actual download
        method _download().
        """
        urls = self._parse_url()
        for url in urls:
            self.url = url
            self._download()
            self.des = self.passed_dir



"""Business set commands."""
import datetime
import typer
from toolbox.common.work import get_month_working_hours, get_month_working_days
from toolbox.common.output import show_message
from toolbox.sets.business import __version__ as business_package_version

app = typer.Typer(short_help="Business CLI Tool")


def version_callback(value: bool):
    """Version callback."""
    if value:
        show_message(f"Business CLI Version: {business_package_version.__version__}")
        raise typer.Exit()


@app.command()
def working_hours(
    is_month: bool = typer.Option(False, '--this-month',
                                  help="Get current month working hours.", is_flag=True)):
    """Get working hours based of a date period."""
    year = datetime.datetime.now().year
    if is_month:
        month = datetime.datetime.now().month
        month_working_hours = get_month_working_hours(month, year)
        show_message(f"Current month working hours: {month_working_hours}")


@app.command()
def working_days(
    is_month: bool = typer.Option(False, '--this-month',
                                  help="Get current month working days.", is_flag=True)):
    """Get working days based of a date period."""
    year = datetime.datetime.now().year
    if is_month:
        month = datetime.datetime.now().month
        month_working_days = get_month_working_days(month, year)
        show_message(f"Current month working days: {month_working_days}")


@app.callback()
def version(ver: bool = typer.Option(None, "--version",
                                     callback=version_callback, help="Get command version.",
                                     is_eager=True)):
    """Version Output."""
    return ver



"""Hashing Algorithms package."""
from enum import Enum


class HashingAlgorithms(str, Enum):
    """Hasing Algorithms enums"""
    SHA_512 = "sha512"
    SHA_256 = "sha256"
    MD5 = "md5"

    @classmethod
    def values(cls) -> list:
        """Get all values from class as list"""
        return list(map(lambda c: c.value, cls))



"""Download set commands."""
from typing import Optional
import typer
from toolbox.common.output import show_message
from toolbox.sets.dw import __version__ as dw_package_version
from toolbox.sets.dw.download import Download

app = typer.Typer(short_help="Download CLI Tool")


def version_callback(value: bool):
    """Version callback."""
    if value:
        show_message(f"Hash CLI Version: {dw_package_version.__version__}")
        raise typer.Exit()


@app.command(short_help="Download from url")
def url(url_link: str,
        destination: Optional[str] = typer.Argument(None,
                                                    help="Target filepath"),
        force: bool = typer.Option(False, '--force', '-f', '-o',
                                   help="Overwrite if the file already exists.", is_flag=True),
        resume: bool = typer.Option(False, '--resume', '-c',
                                    help="Resume failed or cancelled download.", is_flag=True),
        echo: bool = typer.Option(False, '--echo', '-e',
                                  help="Print the filepath to stdout after downloading.",
                                  is_flag=True),
        quiet: bool = typer.Option(False, '--quiet', '-q',
                                   help="Suppress filesize and progress info.", is_flag=True),
        batch: bool = typer.Option(False, '--batch', '-b',
                                   help="Download files in batch.",
                                   is_flag=True)):
    """Download from url."""
    downloader = Download(url=url_link, des=destination, overwrite=force,
                          continue_download=resume, echo=echo,
                          quiet=quiet, batch=batch)
    downloader.download()
    if echo:
        print(downloader.des)


@app.callback()
def version(ver: bool = typer.Option(None, "--version",
                                     callback=version_callback, help="Get command version.",
                                     is_eager=True)):
    """Version Output."""
    return ver



"""Module for file."""
import os
import tomli


def get_filename(file: str) -> str:
    """Return the filename of the file"""
    return os.path.splitext(file)[0]


def load_toml(path: str):
    """Load TOML file"""
    with open(path, mode='rb') as toml_file:
        return tomli.load(toml_file)


def get_full_path(filename: str):
    """Get full path from filename"""
    return os.path.realpath(filename)



"""Module for converting files."""
import os
import ocrmypdf
from pdf2docx import Converter
from toolbox.common.file import get_filename
from toolbox.common.exceptions import ConvertPDFException


def convert_pdf_to_selectable(input_file: str, output_file) -> None:
    """Converts a PDF file to a selectable PDF file.

    Args:
        input_file (str): The path to the PDF file.
        output_file (str): The path to the selectable PDF file.
"""
    input_filename = get_filename(input_file)
    if output_file is None:
        output_file = f"{input_filename}_selectable.pdf"
    try:
        ocrmypdf.ocr(input_file, output_file, deskew=True)
    except Exception as err:
        raise ConvertPDFException(str(err)) from err


def convert_pdf_to_docx(pdf_file: str, docx_file: str):
    """Converts a PDF file to a docx file.

    Args:
        pdf_file (str): The path to the PDF file.
        docx_file (str): The path to the docx file.
    """
    pdf_filename = get_filename(pdf_file)
    if docx_file is None:
        docx_file = f"{pdf_filename}.docx"
    selectable_pdf = f"{pdf_filename}_selectable.pdf"
    convert_pdf_to_selectable(pdf_file, selectable_pdf)
    try:
        converter = Converter(selectable_pdf)
        converter.convert(docx_file, multi_processing=True)
        converter.close()
        os.remove(selectable_pdf)
    except Exception as err:
        raise ConvertPDFException(str(err)) from err



"""Custom Exceptions module."""


class ConvertPDFException(Exception):
    """ConvertPDFException class."""

    def __init__(self, message):
        """ConvertPDFException constructor."""
        super().__init__()
        self.message = message

class DownloadException(Exception):
    """ConvertPDFException class."""

    def __init__(self, message):
        """ConvertPDFException constructor."""
        super().__init__()
        self.message = message



"""Hashing set commands."""
from typing import List, Optional
import hashlib
import typer
from rich.table import Table
from toolbox.common.file import get_full_path
from toolbox.common.output import show_table, show_message
from toolbox.sets.hash.hash_algoritms_enum import HashingAlgorithms
from toolbox.sets.hash import __version__ as hash_package_version

app = typer.Typer(short_help="Hashing CLI Tool")


def version_callback(value: bool):
    """Version callback."""
    if value:
        show_message(f"Hash CLI Version: {hash_package_version.__version__}")
        raise typer.Exit()


@app.command(name="files", short_help="Hashing Files CLI Tool")
def files(hashing_files: Optional[List[str]] = typer.Option(..., '--file'),
          algorithm: HashingAlgorithms = typer.Option(HashingAlgorithms.SHA_512,
                                                      autocompletion=HashingAlgorithms.values)):
    """Hashing files method"""
    table = Table(show_header=True, header_style="bold magenta")
    table.add_column("Algorithm", style="dim", width=12)
    table.add_column("Hash")
    table.add_column("Path")
    for file in hashing_files:
        hasher = hashlib.new(algorithm)
        with open(file, "rb") as hashing_file:
            content = hashing_file.read()
            hasher.update(content)
            table.add_row(algorithm, hasher.hexdigest(), get_full_path(hashing_file.name))
    show_table(table)


@app.callback()
def version(ver: bool = typer.Option(None, "--version",
                                     callback=version_callback,
                                     help="Get command version.",
                                     is_eager=True)):
    """Version Output."""
    return ver



"""Main command."""
import typer
from toolbox.sets.pdf import pdf_app
from toolbox.sets.business import business_app
from toolbox.sets.hash import hash_app
from toolbox.sets.dw import dw_app

app = typer.Typer(short_help="Tools for various automations. 🧰")
app.add_typer(business_app, name="business")
app.add_typer(dw_app, name="dw")
app.add_typer(pdf_app, name="pdf")
app.add_typer(hash_app, name="hash")

command = typer.main.get_command(app)


def main():
    """Main input command."""
    command()


if __name__ == '__main__':
    main()



"""Module for work."""
import datetime


def get_month_working_days(month: int, year: int) -> int:
    """Get the number of working days for a given month.

    Args:
        month (int): Month of the year.
        year (int): Year of the month.
    """
    holidays = {}
    business_days = 0
    for i in range(1, 32):
        try:
            date = datetime.date(year, month, i)
        except ValueError:
            break
        if date.weekday() < 5 and date not in holidays:  # Monday == 0, Sunday == 6
            business_days += 1
    return business_days


def get_month_working_hours(month: int, year: int) -> int:
    """Get the number of working hours for a given month.

    Args:
        month (int): Month of the year.
        year (int): Year of the month.
    """
    return get_month_working_days(month, year) * 8



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['toolbox',
 'toolbox.common',
 'toolbox.sets.business',
 'toolbox.sets.dw',
 'toolbox.sets.hash',
 'toolbox.sets.pdf']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'ocrmypdf>=14.0.1,<15.0.0',
 'pdf2docx>=0.5.6,<0.6.0',
 'pylint>=2.15.5,<3.0.0',
 'pytest>=7.2,<8.0',
 'rich>=12.6.0,<13.0.0',
 'tomli>=2.0.1,<3.0.0',
 'typer[all]>=0.6.1,<0.7.0']

entry_points = \
{'console_scripts': ['test = toolbox.scripts:test',
                     'toolbox = toolbox.cli:main']}

setup_kwargs = {
    'name': 'atelier-toolbox',
    'version': '1.0.2',
    'description': 'Tools for various automations 🧰',
    'long_description': '# atelier-toolbox\n\n![GitHub Workflow Status](https://img.shields.io/github/actions/workflow/status/mihaichris/atelier-toolbox/build.yml)\n![GitHub Workflow Status](https://img.shields.io/github/actions/workflow/status/mihaichris/atelier-toolbox/test.yml?label=test)\n[![Issues](https://img.shields.io/github/issues/mihaichris/atelier-toolbox)](https://github.com/mihaichris/atelier-toolbox/issues)\n[![License](https://img.shields.io/github/license/mihaichris/atelier-toolbox)](https://github.com/mihaichris/atelier-toolbox/blob/main/LICENSE)\n[![Version](https://img.shields.io/github/v/tag/mihaichris/atelier-toolbox)](https://github.com/mihaichris/atelier-toolbox/blob/main/LICENSE)\n[![Python Version](https://img.shields.io/pypi/pyversions/atelier-toolbox)](https://pypi.org/project/atelier-toolbox/)\n\n## Description\n\nTools for various automations 🧰.\n\n## Installation\n\nYou can install the Toolbox from [PyPI](https://pypi.org/):\n\n```python\npython -m pip install atelier-toolbox\n```\nThe package is supported on Python 3.10 and above.\n\n\n## How to use\n\n```python\nUsage: toolbox [OPTIONS] COMMAND [ARGS]...\n\n╭─ Options ─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────╮\n│ --install-completion          Install completion for the current shell.                                                                                                                                   │\n│ --show-completion             Show completion for the current shell, to copy it or customize the installation.                                                                                            │\n│ --help                        Show this message and exit.                                                                                                                                                 │\n╰───────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────╯\n╭─ Commands ────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────╮\n│ business                                                  Business CLI Tool                                                                                                                               │\n│ dw                                                        Download CLI Tool                                                                                                                               │\n│ hash                                                      Hashing Files CLI Tool                                                                                                                          │\n│ pdf                                                       PDF CLI Tool                                                                                                                                    │\n╰───────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────╯\n```\n\n## CHANGELOG\n Please refer to [CHANGELOG.md](https://github.com/mihaichris/toolbox/blob/main/CHANGELOG.md)\n\n## License\n[MIT](https://opensource.org/licenses/MIT)\n',
    'author': 'Mihai',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/mihaichris/atelier-toolbox',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



"""Module for calendar."""
from datetime import datetime, timedelta


def get_first_date_of_month(year, month):
    """Return the first date of the month.

    Args:
        year (int): Year
        month (int): Month

    Returns:
        date (datetime): First date of the current month
    """
    first_date = datetime(year, month, 1)
    return first_date.strftime("%Y-%m-%d")


def get_last_date_of_month(year, month):
    """Return the last date of the month.

    Args:
        year (int): Year, i.e. 2022
        month (int): Month, i.e. 1 for January

    Returns:
        date (datetime): Last date of the current month
    """

    if month == 12:
        last_date = datetime(year, month, 31)
    else:
        last_date = datetime(year, month + 1, 1) + timedelta(days=-1)

    return last_date.strftime("%Y-%m-%d")



"""Module for different scripts."""
import subprocess


def test():
    """
    Run all unittests. Equivalent to:
    `poetry run python -u -m unittest discover`
    """
    subprocess.run(
        ['python', '-u', '-m', 'unittest', 'discover'], check=True
    )
